from __future__ import absolute_import, unicode_literals

from collections import defaultdict


class ValidationErrors(defaultdict):
    """
    Lists of validation errors for each field in a validator.

    Only filled out for a field if the field has an error.
    """
    def __init__(self, errors={}):
        super(ValidationErrors, self).__init__(list)
        self.update(errors)

    def __repr__(self):
        inner = ', '.join('{0}: {1}'.format(k, v) for k, v in self.items())
        return '{' + inner + '}'

    def flatten(self):
        """
        Yield a pair of ``(path, errors)`` for each error.

        >>> list(errors.flatten())
        [
            (['bar'], ['Unknown field']),
            (['foo'], ['This field can not be empty']),
        ]

        If an error is a :cls:`NestedValidatorException`, then errors are
        recursively extracted.  ``path`` will be an array of the path to the
        error. ``errors`` will be an array of the error messages from these
        errors.

        If validator was constructed for a shopping card, which had user
        details and a list of items in a shopping cart, made using a
        NestedValidator inside a ListField, some possible flattened errors
        might be:

        >>> list(errors.flatten())
        [
            (['name'], ['This field can not be empty']),
            (['items', 2, 'quantity'], ['This must be equal to or greater than the minimum of 1']),
        ]
        """
        for name, error_list in sorted(self.items()):
            if isinstance(error_list, NestedValidatorException):
                for nested_name, error in error_list.errors.flatten():
                    yield [name] + nested_name, error
            else:
                yield [name], [error.msg for error in error_list]


class BaseValidationException(Exception):
    """
    All exceptions used by this will be subclasses of this exception class.
    """
    pass


class ValidationException(BaseValidationException):
    """
    Something is wrong!
    """
    def __init__(self, message, **kwargs):
        self.msg = message
        super(ValidationException, self).__init__(message, **kwargs)

    def __str__(self):
        return '{cls}: {msg}'.format(cls=self.__class__.__name__,
                                     msg=self.msg)

    def __repr__(self):
        return '<{str}>'.format(str=self)


class NestedValidatorException(BaseValidationException):
    """
    Something is wrong! Specifically in a nested validator.
    """
    def __init__(self, errors):
        self.errors = errors


class NoData(BaseValidationException):
    """
    Used to indicate that this field had no data supplied. This is different
    from having empty data, which is represented by ``None``. This bypasses the
    bit where data is set in the output dict, so the output dict will **not**
    have the associated key.
    """
    pass



from __future__ import absolute_import, unicode_literals

import copy
import functools

from gettext import gettext as _

from .exceptions import (
    ValidationErrors, ValidationException, NoData, NestedValidatorException)
from .fields import Field


def partition_dict(d, pred, dict_class=dict):
    """
    Split a dict in two based on a predicate. ``d`` is a dict (or dict-like)
    object. ``pred(key, value)`` is a function that returns a bool, which will
    determine which dict the key, value pair will be sent to. Returns two
    dicts, ``(false, true)``, where ``false`` is a dict with all pairs where
    ``pred`` returned ``False``, and ``true`` is a dict with all pairs where
    ``pred`` returned ``True``.
    """
    def iterator(acc, pair):
        f, t = acc
        key, val = pair
        if pred(key, val):
            t[key] = val
        else:
            f[key] = val
        return f, t
    return functools.reduce(iterator, d.items(), (dict_class(), dict_class()))


class DeclarativeFieldsMetaclass(type):
    def __new__(mcs, name, bases, attrs):
        # Split out any fields declared on this Validator
        attrs, new_fields = partition_dict(
            attrs, lambda k, v: isinstance(v, Field))

        # Make the class
        cls = super(DeclarativeFieldsMetaclass, mcs).__new__(
            mcs, name, bases, attrs)

        # Set the declared fields to the `fields` attribute, merging in any
        # existing fields
        fields = {}
        field_sets = [getattr(base, 'fields', {}) for base in reversed(cls.__mro__)]
        field_sets.append(new_fields)
        for field_set in field_sets:
            if field_set is None:
                continue
            for name, field in field_set.items():
                fields[name] = copy.copy(field)
        setattr(cls, 'fields', fields)

        return cls


class BaseValidator(object):
    """
    Validate an input dict against a set of validators, which ensure that the
    data is of the correct input type, format, and value, and return a
    (possibly modified) value for output.
    """

    # A ``dict`` of ``"name": Field()`` entries.
    fields = None

    # Whether or not unknown fields being present in the input data is
    # considered an error.
    allow_unknown_fields = False

    default_error_messages = {
        'unknown': _('Unknown field'),
    }

    def __init__(self, fields=None, allow_unknown_fields=None,
                 error_messages=None):
        if fields is not None:
            self.fields = self.fields.copy()
            self.fields.update(fields)

        if allow_unknown_fields is not None:
            self.allow_unknown_fields = allow_unknown_fields

        messages = {}
        for c in reversed(self.__class__.__mro__):
            messages.update(getattr(c, 'default_error_messages', {}))
        messages.update(error_messages or {})
        self.error_messages = messages

    def clean(self, data):
        """
        Take the form data supplied, and clean it using the validators

        Returns the cleaned and validated data
        """
        errors = ValidationErrors()
        cleaned_data = {}

        # Check for unknown fields
        if not self.allow_unknown_fields:
            for name in data.keys():
                if name not in self.fields:
                    errors[name].append(self.error_messages['unknown'])

        # Validate all incoming fields
        for name, field in self.fields.items():
            try:
                datum = data.get(name, NoData)
                value = field.clean(datum)
                cleaned_data[name] = value

            except NestedValidatorException as err:
                errors[name] = err.msg

            except ValidationException as err:
                errors[name].extend(err.msg)

            except NoData:
                pass

        if errors:
            return None, errors
        else:
            return cleaned_data, None

    def __getitem__(self, key):
        return self.fields[key]


class Validator(BaseValidator, metaclass=DeclarativeFieldsMetaclass):
    pass



import unittest

from valedictory.exceptions import (
    ValidationErrors, ValidationException, NestedValidatorException)


class TestExceptions(unittest.TestCase):
    def test_one_level(self):
        errors = ValidationErrors({
            'foo': [ValidationException("foo error")],
            'bar': [
                ValidationException("bar error 1"),
                ValidationException("bar error 2"),
            ],
        })

        self.assertEqual(list(errors.flatten()), [
            (['bar'], ['bar error 1', 'bar error 2']),
            (['foo'], ['foo error'])])

    def test_nested_dict_errors(self):
        errors = ValidationErrors({
            'foo': [ValidationException("foo error")],
            'bar': NestedValidatorException(ValidationErrors({
                'baz': [ValidationException("bar baz error")],
                'quux': [ValidationException("bar quux error")],
            })),
        })

        self.assertEqual(list(errors.flatten()), [
            (['bar', 'baz'], ['bar baz error']),
            (['bar', 'quux'], ['bar quux error']),
            (['foo'], ['foo error'])])

    def test_nested_list_rrors(self):
        errors = ValidationErrors({
            'foo': [ValidationException("foo error")],
            'bar': NestedValidatorException(ValidationErrors({
                1: [ValidationException("bar 1 error")],
                3: [ValidationException("bar 3 error")],
            })),
        })

        self.assertEqual(list(errors.flatten()), [
            (['bar', 1], ['bar 1 error']),
            (['bar', 3], ['bar 3 error']),
            (['foo'], ['foo error'])])



import datetime
import unittest

from valedictory import Validator

from valedictory.exceptions import (
    ValidationException, NestedValidatorException, NoData)

from valedictory.fields import (
    Field, StringField, BooleanField, IntegerField, EmailField, DateField,
    YearMonthField, ChoiceField, DigitField, CreditCardField,
    ListField, NestedValidator)


class TestField(unittest.TestCase):

    def test_required(self):
        field = Field()
        self.assertEqual("hello", field.clean("hello"))
        self.assertEqual("", field.clean(""))

        with self.assertRaises(ValidationException):
            field.clean(NoData)

    def test_not_required(self):
        field = StringField(required=False)
        self.assertEqual("hello", field.clean("hello"))
        self.assertEqual("", field.clean(""))

        with self.assertRaises(NoData):
            field.clean(NoData)


class TestStringField(unittest.TestCase):

    def test_simple(self):
        field = StringField()
        self.assertEqual("hello", field.clean("hello"))
        with self.assertRaises(ValidationException):
            self.assertEqual("", field.clean(""))

    def test_min_length(self):
        field = StringField(min_length=3)

        self.assertEqual("hello", field.clean("hello"))
        with self.assertRaises(ValidationException):
            field.clean("no")

    def test_max_length(self):
        field = StringField(max_length=3)

        self.assertEqual("hi", field.clean("hi"))
        with self.assertRaises(ValidationException):
            field.clean("hello")

    def test_not_required(self):
        field = StringField(required=False)
        self.assertEqual("", field.clean(""))
        with self.assertRaises(NoData):
            field.clean(NoData)

    def test_invalid_type(self):
        field = StringField()
        with self.assertRaises(ValidationException):
            field.clean(10)


class TestBooleanField(unittest.TestCase):

    def test_simple(self):
        field = BooleanField()
        self.assertEqual(True, field.clean(True))
        self.assertEqual(False, field.clean(False))

    def test_invalid_type(self):
        field = BooleanField()
        with self.assertRaises(ValidationException):
            field.clean(0)
        with self.assertRaises(ValidationException):
            field.clean(1)
        with self.assertRaises(ValidationException):
            field.clean("")
        with self.assertRaises(ValidationException):
            field.clean("True")
        with self.assertRaises(ValidationException):
            field.clean("true")


class TestIntegerField(unittest.TestCase):

    def test_simple(self):
        field = IntegerField()
        self.assertEqual(-10, field.clean(-10))
        self.assertEqual(0, field.clean(0))
        self.assertEqual(42, field.clean(42))

    def test_floats(self):
        field = IntegerField()
        with self.assertRaises(ValidationException):
            field.clean(1.0)
        with self.assertRaises(ValidationException):
            field.clean(3.14159264)

    def test_invalid_type(self):
        field = IntegerField()
        with self.assertRaises(ValidationException):
            field.clean("Hello")
        with self.assertRaises(ValidationException):
            field.clean("10")
        with self.assertRaises(ValidationException):
            field.clean(True)


class TestEmailField(unittest.TestCase):

    def test_simple(self):
        field = EmailField()
        self.assertEqual("test@example.com", field.clean("test@example.com"))
        # Address validation is not perfect...
        self.assertEqual("t@e.c", field.clean("t@e.c"))

    def test_max_length(self):
        field = EmailField(max_length=10)
        self.assertEqual("t@e.c", field.clean("t@e.c"))
        with self.assertRaises(ValidationException):
            field.clean("test@example.com")

    def test_invalid_address(self):
        field = EmailField()
        with self.assertRaises(ValidationException):
            field.clean("t@e")
        with self.assertRaises(ValidationException):
            field.clean("@te.com")
        with self.assertRaises(ValidationException):
            field.clean("test@")
        with self.assertRaises(ValidationException):
            field.clean("te.com")
        with self.assertRaises(ValidationException):
            field.clean("test")

    def test_invalid_type(self):
        field = EmailField()
        with self.assertRaises(ValidationException):
            field.clean(10)


class TestDateField(unittest.TestCase):

    def test_simple(self):
        field = DateField()
        self.assertEqual(
            datetime.date(1989, 10, 16),
            field.clean("1989-10-16"))
        self.assertEqual(
            datetime.date(2345, 6, 7),
            field.clean("2345-06-7"))

    def test_invalid_dates(self):
        field = EmailField(max_length=10)

        # No leap year this year
        with self.assertRaises(ValidationException):
            field.clean("2015-2-29")

        # Too many numbers in the year.
        # Sorry, Long Now Foundation!
        with self.assertRaises(ValidationException):
            field.clean("10000-01-01")

        # Small dates dont work either. Anything before 1582 is a bit suss in
        # the Gregorian calendar anyway.
        with self.assertRaises(ValidationException):
            field.clean("999-01-01")

        # wat r u doin?
        with self.assertRaises(ValidationException):
            field.clean("Not even a date")


class TestYearMonthField(unittest.TestCase):

    def test_simple(self):
        field = YearMonthField()
        self.assertEqual(
            (1989, 10),
            field.clean("1989-10"))
        self.assertEqual(
            (2345, 6),
            field.clean("2345-06"))

    def test_invalid_dates(self):
        field = EmailField(max_length=10)

        # Too many numbers in the year.
        # Sorry, Long Now Foundation!
        with self.assertRaises(ValidationException):
            field.clean("10000-01")

        # Small dates dont work either. Sorry.
        with self.assertRaises(ValidationException):
            field.clean("999-01")

        # wat r u doin?
        with self.assertRaises(ValidationException):
            field.clean("nope-no")


class TestChoiceField(unittest.TestCase):

    def test_simple(self):
        choices = ["hello", 10, True, False, None]
        field = ChoiceField(choices)
        for choice in choices:
            self.assertEqual(
                choice,
                field.clean(choice))

    def test_invalid(self):
        choices = ["hello", 10, True, False, None]
        invalid_choices = ["nope", 11, {}, []]
        field = ChoiceField(choices)
        for choice in invalid_choices:
            with self.assertRaises(ValidationException):
                field.clean(choice)


class TestDigitField(unittest.TestCase):

    def test_simple(self):
        field = DigitField()
        self.assertEqual(
            "1234567890",
            field.clean("1234567890"))
        self.assertEqual(
            "000",
            field.clean("000"))
        self.assertEqual(
            "01020",
            field.clean("01020"))

    def test_invalid(self):
        field = DigitField()
        with self.assertRaises(ValidationException):
            field.clean("hello")
        with self.assertRaises(ValidationException):
            field.clean("123abc")
        with self.assertRaises(ValidationException):
            field.clean("abc123")


class TestCreditCardField(unittest.TestCase):

    def test_simple(self):
        field = CreditCardField()
        self.assertEqual(
            "5123456789012346",
            field.clean("5123 4567 8901 2346"))
        self.assertEqual(
            "4111111111111111",
            field.clean("4111-1111-1111-1111"))
        self.assertEqual(
            "378282246310005",
            field.clean("3 78 2-82 2--46  3 -10- 005"))

    def test_invalid_numbers(self):
        field = CreditCardField()
        with self.assertRaises(ValidationException):
            field.clean('5123_4567_8901_2346')
        with self.assertRaises(ValidationException):
            field.clean('5111111111111111')


class TestListField(unittest.TestCase):
    def test_string_list(self):
        field = ListField(StringField())

        self.assertEqual(
            ["foo", "bar", "baz"],
            field.clean(["foo", "bar", "baz"]))

        self.assertEqual([], field.clean([]))

    def test_int_list(self):
        field = ListField(IntegerField())

        self.assertEqual(
            [1, 2, 3, 4],
            field.clean([1, 2, 3, 4]))

        self.assertEqual([], field.clean([]))

    def test_bad_list(self):
        """
        Errors should be indexed by their index in the list
        """
        field = ListField(IntegerField())

        try:
            field.clean([1, "nope", 3, "strings"])
        except NestedValidatorException as e:
            self.assertEqual([1, 3], sorted(e.errors.keys()))
        else:
            self.fail("Expecting to catch ValidationException")


class TestNestedValidators(unittest.TestCase):
    def test_nested_validator(self):
        field = NestedValidator(Validator(fields={
            'int': IntegerField(),
            'string': StringField()}))

        self.assertEqual(
            {'int': 10, 'string': 'foo'},
            field.clean({'string': 'foo', 'int': 10}))


class TestNestedLists(unittest.TestCase):
    def test_nested_lists(self):
        field = ListField(NestedValidator(Validator(fields={
            'int': IntegerField(), 'string': StringField()})))

        data = [{'int': i, 'string': s}
                for i, s in zip(range(3), 'foo bar baz'.split())]
        self.assertEqual(
            data,
            field.clean(data))



from __future__ import absolute_import, unicode_literals

from django.core.files.uploadedfile import UploadedFile
from django.db.models import Model

from valedictory.exceptions import ValidationException
from valedictory.fields import TypedField


class UploadedFileField(TypedField):
    """
    Accepts uploaded files
    """
    required_types = UploadedFile
    type_name = 'file'


class ForeignKeyField(TypedField):
    """
    A field that accepts foreign keys to a Django model.
    """
    type_name = 'foreign key'

    def __init__(self, queryset, field='pk', key_type=int, **kwargs):
        super(ForeignKeyField, self).__init__(**kwargs)
        """
        Construct a new ForeignKeyField

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * The models that the field can reference is set by ``queryset``.
          If a model is supplied, all objects of that type are allowed.
        * ``field`` is the name of the model field to search on. Defaults to
          ``pk``.
        * ``key_type`` is the type of the field being searched. Defaults to
          ``int``
        """
        if isinstance(queryset, Model):
            queryset = queryset.objects.all()
        self.queryset = queryset
        self.field = field
        self.required_types = key_type

    def clean(self, name, data):
        value = super(ForeignKeyField, self).clean(name, data)
        queryset = self.queryset
        model = queryset.model
        try:
            return queryset.get(**{self.field: value})
        except model.DoesNotExist:
            raise ValidationException("Object does not exist")
        except model.MultipleObjectsReturned:
            return ValidationException("Multiple objects returned")

    def __copy__(self, **kwargs):
        return super(ForeignKeyField, self).__copy__(
            queryset=self.queryset, field=self.field,
            key_type=self.required_types)



import datetime
import re

from gettext import gettext as _

from .exceptions import (
    ValidationErrors, BaseValidationException, ValidationException, NoData,
    NestedValidatorException)


class Field(object):
    required = True

    default_error_messages = {
        "required": _("This field is required"),
    }

    def __init__(self, required=True, error_messages=None):
        self.required = required

        messages = {}
        for c in reversed(self.__class__.__mro__):
            messages.update(getattr(c, 'default_error_messages', {}))
        messages.update(error_messages or {})
        self.error_messages = messages

    def clean(self, data):
        if data is NoData:
            if self.required:
                raise ValidationException(self.error_messages['required'])
            else:
                raise NoData
        return data

    def __copy__(self, **kwargs):
        return self.__class__(required=self.required, **kwargs)


class TypedField(Field):
    """
    Base Field class for fields that require a specific type of input, such as
    strings, integers, or booleans.
    """
    required_types = ()
    excluded_types = ()
    type_name = u''

    default_error_messages = {
        "invalid_type": _("This field must be a {type}, not {actual_type}"),
    }

    def clean(self, data):
        value = super(TypedField, self).clean(data)

        if (not isinstance(value, self.required_types) or
                isinstance(value, self.excluded_types)):
            actual_type = type(value)
            raise ValidationException(
                self.error_messages["invalid_type"].format(
                    type=self.type_name,
                    actual_type=actual_type.__name__))

        return value


class StringField(TypedField):
    """
    Accepts a string, and only strings.
    """
    required_types = str
    type_name = u'string'
    min_length = 0
    max_length = float('inf')

    default_error_messages = {
        "non_empty": "This field can not be empty",
        "min_length": "Minimum length {0}",
        "max_length": "Maximum length {0}",
    }

    def __init__(self, min_length=None, max_length=None, **kwargs):
        """
        Construct a field that accepts only strings.

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * The ``min_length`` and ``max_length`` keyword arguments set the
          minimum and maximum length of string the field will accept.
        """
        super(StringField, self).__init__(**kwargs)

        if min_length is not None:
            self.min_length = min_length
        if max_length is not None:
            self.max_length = max_length

    def clean(self, data):
        value = super(StringField, self).clean(data)
        if value == u'' and self.required:
            raise ValidationException(self.error_messages['required'])

        if len(value) < self.min_length:
            if self.min_length == 1:
                raise ValidationException(self.error_messages['non_empty'])
            else:
                raise ValidationException(
                    self.error_messages['min_length'].format(self.min_length))

        if len(value) > self.max_length:
            raise ValidationException(
                self.error_messages['max_length'].format(self.max_length))

        return value

    def __copy__(self, **kwargs):
        return super(StringField, self).__copy__(min_length=self.min_length,
                                                 max_length=self.max_length)


class BooleanField(TypedField):
    """
    A field that only accepts True and False values.
    """
    required_types = bool
    type_name = u'boolean'


class IntegerField(TypedField):
    """
    A field that only accepts integer values.
    """
    required_types = int
    excluded_types = bool  # bools subclass ints :(
    type_name = u'integer'
    default_error_messages = {
        'min_value': _("This must be equal to or greater than the minimum of {0}"),
        'max_value': _("This must be equal to or less than the maximum of {0}"),
    }

    def __init__(self, min=None, max=None, *args, **kwargs):
        """
        Construct a field that accepts only integers.

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * The ``min`` and ``max`` keyword arguments set the minimum and maximum
          value the field will accept. The range is inclusive.
        """
        super(IntegerField, self).__init__(*args, **kwargs)
        self.min = min
        self.max = max

    def clean(self, data):
        value = super(IntegerField, self).clean(data)

        if self.min is not None and value < self.min:
            raise ValidationException(
                self.error_messages['min_value'].format(self.min))

        if self.max is not None and value > self.max:
            raise ValidationException(
                self.error_messages['max_value'].format(self.max))

        return value

    def __copy__(self, **kwargs):
        return super(IntegerField, self).__copy__(min=self.min,
                                                  max=self.max)


class EmailField(StringField):
    """
    A field that only accepts email address strings. The email matching regular
    expression only checks for basic conformance: the string must have at least
    one character, then an '@' symbol, then more characters with at least one
    dot.
    """

    email_re = re.compile(r"[^@]+@[^@]+\.[^@]+")

    default_error_messages = {
        'invalid_email': _("Not a valid email address"),
    }

    def clean(self, data):
        value = super(EmailField, self).clean(data)
        if not self.email_re.match(value):
            raise ValidationException(self.error_messages['invalid_email'])
        return value


class DateField(StringField):
    """
    A field that only accepts ISO 8601 date strings.

    After cleaning, a ``datetime.date`` instance is returned.
    """

    # YYYY-MM-DD = 10 chars.
    max_length = 10

    default_error_messages = {
        'invalid_date': _("Not a valid date"),
    }

    def clean(self, data):
        date_string = super(DateField, self).clean(data)

        try:
            date = datetime.datetime.strptime(date_string, "%Y-%m-%d").date()
        except ValueError:
            raise ValidationException(self.error_messages['invalid_date'])

        return date


class YearMonthField(StringField):
    """
    A field that only accepts ``YYYY-MM`` date strings.

    After cleaning, a tuple of ``(year, month)`` instance is returned.
    """

    # YYYY-MM = 9 chars.
    # Support 6 digit years, to keep the long now/y10k people happy
    max_length = 9

    default_error_messages = {
        'invalid_date': _("Not a valid date"),
    }

    def clean(self, data):
        date_string = super(YearMonthField, self).clean(data)

        try:
            date = datetime.datetime.strptime(date_string, "%Y-%m").date()
        except ValueError:
            raise ValidationException(self.error_messages['invalid_date'])

        return (date.year, date.month)


class ChoiceField(Field):
    """
    A field that only accepts values from a predefined list. The values can be
    of any type.
    """

    default_error_messages = {
        'invalid_choice': _("Not a valid choice"),
    }

    def __init__(self, choices, **kwargs):
        """
        Construct a ChoiceField.

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * ``choices`` must be an iterable of choices. The field will only
          accept data if the value is in this list.
        """
        super(ChoiceField, self).__init__(**kwargs)
        self.choices = list(choices)

    def clean(self, data):
        value = super(ChoiceField, self).clean(data)
        if value not in self.choices:
            raise ValidationException(self.error_messages['invalid_choice'])
        return value

    def __copy__(self, **kwargs):
        return super(ChoiceField, self).__copy__(choices=self.choices)


class PunctuatedCharacterField(TypedField):
    """
    A field that accepts characters only from an alphabet of allowed
    characters. A set of allowed punctuation characters are allowed and
    discarded when cleaned.
    """
    required_types = (str)
    type_name = u'string'

    alphabet = None
    punctuation = None

    min_length = 0
    max_length = float('inf')

    default_error_messages = {
        "allowed_characters": _("Only the characters '{alphabet}{punctuation}' are allowed"),
        "min_length": "Minimum length {0}",
        "max_length": "Maximum length {0}",
    }

    def __init__(self, alphabet=None, punctuation=None,
                 min_length=None, max_length=None, **kwargs):
        """
        Construct a new PunctuatedCharacterField.

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * ``alphabet`` is a string of allowed characters.
        * ``puctuation`` is a string of allowed punctuation characters that
          will be stripped as part of validation.
        * ``min_length`` and ``max_length`` are the minimum and maximum allowed
          number of characters. Length is calculated *after* punctuation
          characters are removed.
        """
        super(PunctuatedCharacterField, self).__init__(**kwargs)

        if alphabet is not None:
            self.alphabet = str(alphabet)
        if punctuation is not None:
            self.punctuation = str(punctuation)

        if min_length is not None:
            self.min_length = min_length
        if max_length is not None:
            self.max_length = max_length

    def clean(self, data):
        value = super(PunctuatedCharacterField, self).clean(data)

        # Strip out punctuation
        alphabet_dict = dict((ord(c), None) for c in self.alphabet)
        punctuation_dict = dict((ord(c), None) for c in self.punctuation)

        value = value.translate(punctuation_dict)
        stripped_value = value.translate(alphabet_dict)

        if len(stripped_value) != 0:
            raise ValidationException(
                self.error_messages['allowed_characters'].format(
                    alphabet=self.alphabet,
                    punctuation=self.punctuation))

        if len(value) < self.min_length:
            raise ValidationException(
                self.error_messages['min_length'].format(self.min_length))

        if len(value) > self.max_length:
            raise ValidationException(
                self.error_messages['max_length'].format(self.max_length))

        return value

    def __copy__(self, **kwargs):
        return super(PunctuatedCharacterField, self).__copy__(
            alphabet=self.alphabet, punctuation=self.punctuation,
            min_length=self.min_length, max_length=self.max_length)


class RestrictedCharacterField(PunctuatedCharacterField):
    """
    A field that only allows a defined alphabet of characters to be used
    """
    punctuation = ''


class DigitField(RestrictedCharacterField):
    """
    A field that only allows digits. It is not treated as a number, however,
    and leading zeros are preserved
    """
    alphabet = '0123456789'


class CreditCardField(PunctuatedCharacterField):
    """
    Accepts credit card numbers
    """
    punctuation = ' -'
    alphabet = '0123456789'

    min_length = 12
    max_length = 20

    default_error_messages = {
        "luhn_checksum": _("The credit card number is not valid"),
    }

    def clean(self, data):
        value = super(CreditCardField, self).clean(data)

        if not self.luhn_checksum(value):
            raise ValidationException(self.error_messages['luhn_checksum'])

        return value

    def luhn_checksum(self, card_number):
        digits = list(map(int, reversed(card_number)))
        evens = sum(digits[0::2])
        odds = sum(sum(divmod(digit * 2, 10)) for digit in digits[1::2])
        return (evens + odds) % 10 == 0


class ListField(TypedField):
    """
    List fields allow checking that all items in a list all pass
    validation.
    """

    required_types = list
    type_name = 'list'

    def __init__(self, field, **kwargs):
        """
        Construct a new ListField

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * ``field`` should be an instance of a Field subclass. Each item in the
          submitted list will be validated and cleaned with Field.
        """
        super(ListField, self).__init__(**kwargs)
        self.field = field

    def clean(self, data):
        value = super(ListField, self).clean(data)

        errors = ValidationErrors()
        cleaned_list = []
        for i, datum in enumerate(value):
            try:
                cleaned_list.append(self.field.clean(datum))
            except BaseValidationException as err:
                errors[i].append(err)

        if errors:
            raise NestedValidatorException(errors)
        return cleaned_list


class NestedValidator(TypedField):
    """
    Nested validators allow nesting dicts inside one another. A validator is
    used to validate and clean the nested dict.
    """
    required_types = (dict, )
    type_name = 'object'

    def __init__(self, validator, **kwargs):
        """
        Construct a new NestedValidator

        In addition to the arguments accepted by the ``Field`` class, the
        following arguments are accepted:

        * ``validator`` should be an instance of a Validator class. The nested
          dict is passed to this validator for validation and cleaning.
        """
        super(NestedValidator, self).__init__(**kwargs)
        self.validator = validator

    def clean(self, data):
        value = super(NestedValidator, self).clean(data)

        cleaned_data, errors = self.validator.clean(value)
        if errors:
            raise NestedValidatorException(errors)
        return cleaned_data

    def __copy__(self, **kwargs):
        return super(NestedValidator, self).__copy__(validator=self.validator)



import unittest

from valedictory import Validator, fields
from valedictory.base import partition_dict


class TestValidators(unittest.TestCase):

    def test_cleaning_fields(self):
        validator = Validator(fields={
            'int': fields.IntegerField(),
            'string': fields.StringField()})

        data = {'int': 10, 'string': 'foo'}
        cleaned_data, errors = validator.clean(data)

        self.assertEqual(data, cleaned_data)
        self.assertIsNone(errors)

    def test_unknown_fields_disallowed(self):
        """
        Unknown fields should throw an error
        """
        validator = Validator(fields={
            'int': fields.IntegerField(),
            'string': fields.StringField()})

        data = {'int': 10, 'string': 'foo', 'nope': 'nope'}
        cleaned_data, errors = validator.clean(data)
        self.assertEqual(['nope'], sorted(errors.keys()))
        self.assertIsNone(cleaned_data)

    def test_unknown_fields_allowed(self):
        """
        Unknown fields can be silently allowed by setting a flag
        """
        validator = Validator(allow_unknown_fields=True, fields={
            'int': fields.IntegerField(),
            'string': fields.StringField()})

        cleaned_data, errors = validator.clean({'int': 10, 'string': 'foo', 'nope': 'nope'})

        self.assertEqual({'int': 10, 'string': 'foo'}, cleaned_data)
        self.assertIsNone(errors)


class TestDeclarativeValidators(unittest.TestCase):

    def test_no_inheritance(self):
        class MyValidator(Validator):
            int = fields.IntegerField()
            string = fields.StringField()

        self.assertEqual(
            sorted(MyValidator.fields.keys()),
            ['int', 'string'])

        data = {'int': 10, 'string': 'foo'}
        cleaned_data, errors = MyValidator().clean(data)
        self.assertEqual(data, cleaned_data)
        self.assertIsNone(errors)

    def test_single_inheritance(self):
        class ParentValidator(Validator):
            int = fields.IntegerField()

        class ChildValidator(ParentValidator):
            string = fields.StringField()

        self.assertEqual(
            sorted(ChildValidator.fields.keys()),
            ['int', 'string'])

    def test_multiple_inheritance(self):
        class P1Validator(Validator):
            int = fields.IntegerField()
            override = fields.BooleanField()

        class P2Validator(Validator):
            string = fields.StringField()
            override = fields.DateField()

        class ChildValidator(P1Validator, P2Validator):
            email = fields.EmailField()

        self.assertEqual(
            sorted(ChildValidator.fields.keys()),
            sorted(['email', 'int', 'override', 'string']))

        # Field overriding should occur in MRO order, so override should have
        # come from P1Validator
        self.assertIsInstance(ChildValidator.fields['override'],
                              fields.BooleanField)

    def test_methods_and_attributes(self):
        class MyValidator(Validator):
            int = fields.IntegerField()
            string = fields.StringField()

            bar = "baz"

            def do_thing(self):
                return "foo"

        self.assertEqual(
            sorted(MyValidator.fields.keys()),
            ['int', 'string'])

        validator = MyValidator()
        data = {'int': 10, 'string': 'foo'}
        cleaned_data, errors = validator.clean(data)

        self.assertEqual(data, cleaned_data)
        self.assertIsNone(errors)
        self.assertEqual("foo", validator.do_thing())
        self.assertEqual("baz", validator.bar)


class TestMisc(unittest.TestCase):
    def test_partition_dict(self):
        keys = set()
        values = set()

        def pred(key, value):
            keys.add(key)
            values.add(value)
            return key % 2 == 0

        d = {i: chr(i + 65) for i in range(10)}
        odd, even = partition_dict(d, pred)

        self.assertEqual(odd, {1: 'B', 3: 'D', 5: 'F', 7: 'H', 9: 'J'})
        self.assertEqual(even, {0: 'A', 2: 'C', 4: 'E', 6: 'G', 8: 'I'})
        self.assertEqual(keys, set(range(10)))
        self.assertEqual(values, set("ABCDEFGHIJ"))



#!/usr/bin/env python3
"""
Install valedictory using setuptools
"""

from setuptools import setup, find_packages

from valedictory import __version__

with open('README.rst', 'r') as f:
    readme = f.read()

setup(
    name='valedictory',
    version=__version__,
    description='Validate dicts against a schema',
    long_description=readme,
    author='Tim Heap',
    author_email='tim@timheap.me',
    url='https://bitbucket.org/tim_heap/valedictory/',

    install_requires=[],
    zip_safe=False,
    license='BSD License',

    packages=find_packages(),

    include_package_data=True,
    package_data={},

    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
    ],
)
